//TODO Later: check that the worldbase agent for the request exists

#ifndef CHECK_DATABASE_CONSISTENCY_H
#define CHECK_DATABASE_CONSISTENCY_H

#include <boost/regex.hpp>

#include <hatp_msgs/WorldBase.h> 
#include <hatp_msgs/Entity.h>
#include <hatp_msgs/Attribute.h>

#include <hatp_msgs/TaskDescription.h>

struct DatabaseChecker{
	static bool checkEntityName(std::string const& name){
		static const boost::regex regex("[A-Z][A-Za-z0-9_]*");
		return regex_match(name, regex);
	}

	static bool checkAttributeName(std::string const& name){
		static const boost::regex regex("[a-z][A-Za-z0-9]*");
		return regex_match(name, regex);
	}

	//String values must match a certain type
	static bool isValidStringValue(std::string const& string){
		static const boost::regex regex("[a-zA-Z0-9_./-]*");
		return regex_match(string, regex);
	}

	static bool isValidNumberValue(std::string const& number){
		static const boost::regex regex("[-+]?([0-9]*[.])?[0-9]+");
		return regex_match(number, regex);
	}

	static bool isValidBoolValue(std::string const& value){
		return (value=="true" || value=="false" || value=="TRUE" || value=="FALSE" || value=="True" || value=="False" || value=="1" || value=="0");
	}
	
	static bool checkTaskName(std::string const& name){
		return checkEntityName(name);
	}
	
	static bool checkTaskParameter(std::string const& param){
		static const boost::regex regex("[A-Z][A-Za-z0-9_]* [A-Z][A-Za-z0-9_]*");
		return regex_match(param, regex);
	}

//TODO 	static bool checkDatabaseConsistency(hatp_msgs::WorldBase const& database, bool silentMode=true){
	static bool checkFactDatabaseConsistency(hatp_msgs::WorldBase& database, bool silentMode=true){
		std::map<std::string, std::string> entities; //<name, type>
		std::set<std::string> possibleEntityTypes;
		struct attributeInfo{
			std::string type;
			bool dynamic;
			bool set;
			std::string previousDeclarationInEntity;
		};
		typedef std::map< std::pair<std::string, std::string>, attributeInfo > attributeType;
		attributeType attributes; //< < entityType, attributeName >, attributeInfo >
		
		if(database.agent.empty()){
			//First retrieve all the entities - to get the different possible entities and their types
			for(std::vector<hatp_msgs::Entity>::const_iterator it=database.entities.begin(); it!=database.entities.end(); ++it){
				if(!checkEntityName(it->name)){
					if(!silentMode) ROS_ERROR("Entity %s has not a valid name", it->name.c_str());
					return false;
				}
				if(!checkEntityName(it->type)){
					if(!silentMode) ROS_ERROR("Entity %s has not a valid type: %s", it->name.c_str(), it->type.c_str());
					return false;
				}
				if(entities.insert(std::make_pair(it->name, it->type)).second==false){
					if(!silentMode) ROS_ERROR("Entity %s already exsits", it->name.c_str());
					return false;
				}
				possibleEntityTypes.insert(it->type);
			}
			
			//Now it is possible to check the rest of the database
//TODO - PUT BACK	for(std::vector<hatp_msgs::Entity>::const_iterator itEntity=database.entities.begin(); itEntity!=database.entities.end(); ++itEntity){
			for(std::vector<hatp_msgs::Entity>::iterator itEntity=database.entities.begin(); itEntity!=database.entities.end(); ++itEntity){
//TODO - PUT BACK 		for(std::vector<hatp_msgs::Attribute>::const_iterator itAttr=itEntity->attributes.begin(); itAttr!=itEntity->attributes.end(); ++itAttr){
				for(std::vector<hatp_msgs::Attribute>::iterator itAttr=itEntity->attributes.begin(); itAttr!=itEntity->attributes.end(); ++itAttr){
					if(!checkAttributeName(itAttr->name)){
						if(!silentMode) ROS_ERROR("Entity attribute %s.%s has not a valid name", itEntity->name.c_str(), itAttr->name.c_str());
						return false;
					}
					
					//Check that the attribute type is valid
					if(itAttr->type!="string" && itAttr->type!="bool" && itAttr->type!="number"){
						//TODO FIX LATER - THE TYPE IS NOT STORED IN THE INITWORLDBASE ... FIND A SOLUTION
// 						if(!checkEntityName(itAttr->type) || possibleEntityTypes.find(itAttr->type)==possibleEntityTypes.end()){
// 							if(!silentMode) ROS_ERROR("Entity attribute %s.%s has not a valid type: %s", itEntity->name.c_str(), itAttr->name.c_str(), itAttr->type.c_str());
// 							return false;
// 						}
					}
					
					//Check that it does not already exists, if so check consistency
					attributeType::const_iterator itTemp=attributes.find(std::make_pair(itEntity->type, itAttr->name));
					if(itTemp!=attributes.end()){
						//This attribute type already exists, ensure it is consistent with previous declarations
						if(itTemp->second.type != itAttr->type){
							if(itAttr->type==""){ //TODO - FIX LATER - SEE ABOVE
								itAttr->type=itTemp->second.type;
							}else
							if(!silentMode){ ROS_ERROR("Entity attribute %s.%s of type %s does not match previous declaration in %s.%s of type %s", itEntity->name.c_str(), itAttr->name.c_str(), itAttr->type.c_str(), itTemp->second.previousDeclarationInEntity.c_str(), itTemp->first.second.c_str(), itTemp->second.type.c_str());
							return false;
							}
						}
						if(itTemp->second.dynamic != (itAttr->dynamic!=0)){
							if(!silentMode) ROS_ERROR("Entity attribute %s.%s of type %s does not match previous declaration in %s.%s of type %s", itEntity->name.c_str(), itAttr->name.c_str(), (itAttr->dynamic?"dynamic":"static"), itTemp->second.previousDeclarationInEntity.c_str(), itTemp->first.second.c_str(), (itTemp->second.dynamic?"dynamic":"static"));
							return false;
						}
					}else{
						attributes.insert(std::make_pair(std::make_pair(itEntity->type, itAttr->name), attributeInfo({itAttr->type, (itAttr->dynamic!=0), (itAttr->set!=0), itEntity->name})));
					}
					
					//Check that the type and value are valid
					//First check that the number of values is valid (<=1 if atom)
					if(itAttr->set==0){
						if(itAttr->values.size()>1){
							if(!silentMode) ROS_ERROR("Entity attribute %s.%s is declared as an atom, but the value field contains more than one value", itEntity->name.c_str(), itAttr->name.c_str());
							return false;
						}
					}
					
					for(std::vector<std::string>::const_iterator itVal=itAttr->values.begin(); itVal!=itAttr->values.end(); ++itVal){
						//If the attribute is an atom, and the value is "" then it is skipped (treated as empty), otherwise it has to be tested
						if(itAttr->set==0 && (*itVal)=="") continue;
						
						if(itAttr->type=="string"){
							if(!isValidStringValue(*itVal)){
								if(!silentMode) ROS_ERROR("Entity attribute %s.%s is declared as a string, but the value \"%s\" is not a valid string", itEntity->name.c_str(), itAttr->name.c_str(), itVal->c_str());
								return false;
							}
						}else if(itAttr->type=="bool"){
							if(!isValidBoolValue(*itVal)){
								if(!silentMode) ROS_ERROR("Entity attribute %s.%s is declared as a boolean, but the value \"%s\" is not a valid boolean", itEntity->name.c_str(), itAttr->name.c_str(), itVal->c_str());
								return false;
							}
						}else if(itAttr->type=="number"){
							if(!isValidNumberValue(*itVal)){
								if(!silentMode) ROS_ERROR("Entity attribute %s.%s is declared as a number, but the value \"%s\" is not a valid number", itEntity->name.c_str(), itAttr->name.c_str(), itVal->c_str());
								return false;
							}
						}else{
							if(!checkEntityName(*itVal)){
								if(!silentMode) ROS_ERROR("Entity attribute %s.%s is declared as an entity, but the value \"%s\" is not a valid entity name", itEntity->name.c_str(), itAttr->name.c_str(), itVal->c_str());
								return false;
							}
							//Also check the the entity exists
							if(entities.find(*itVal)==entities.end()){
								if(!silentMode) ROS_ERROR("Entity attribute %s.%s is declared as an entity, but the entity \"%s\" does not exist", itEntity->name.c_str(), itAttr->name.c_str(), itVal->c_str());
								return false;
							}
						}
					}
				}
			}
			
		}else{
			if(!silentMode) ROS_INFO("Database for agent: %s", database.agent.c_str());
			if(!silentMode) ROS_ERROR("Not implemented yet !");
		}
		
		return true;
	}

// 	static bool checkDatabaseConsistency(hatp_msgs::WorldBase::ConstPtr const& database, bool silentMode=true){
	static bool checkFactDatabaseConsistency(hatp_msgs::WorldBase::Ptr& database, bool silentMode=true){
		if(!database) return false;
		return checkFactDatabaseConsistency(*database, silentMode);
	}
	
	static bool checkHTNDatabaseConsistency(std::vector<hatp_msgs::TaskDescription> const& database, bool silentMode=true){
		std::vector<hatp_msgs::TaskDescription>::const_iterator itTask;
		std::set<std::string> declaredTasks;
		
		for(itTask=database.begin(); itTask!=database.end(); ++itTask){
			if(!checkTaskName(itTask->name)){
				if(!silentMode) ROS_ERROR("Task %s has an invalid name", itTask->name.c_str());
				return false;
			}
			
			std::vector<std::string>::const_iterator itParam;
			for(itParam=itTask->parameters.begin(); itParam!=itTask->parameters.end(); ++itParam){
				if(!checkTaskParameter(*itParam)){
					if(!silentMode) ROS_ERROR("Task %s has an invalid parameter: \"%s\", must be a pair of the form \"Type Name\".", itTask->name.c_str(), itParam->c_str());
					return false;
				}
			}
			
			//If the task is valid add it to the list of task decalred, failure if it already exists
			if(!declaredTasks.insert(itTask->name).second){
				if(!silentMode) ROS_ERROR("Task %s has already been declared", itTask->name.c_str());
				return false;
			}
		}
		return true;
	}
};

#endif //CHECK_DATABASE_CONSISTENCY_H