#ifndef _HATPTASKLIST_H_
#define _HATPTASKLIST_H_

#include <map>
#include <memory>

#include <libhatp/hatpTask.h>

class hatpPlan;

class hatpTaskList{
private:
	hatpTaskMap tasks;
	
	hatpTaskList(hatpTaskList const&);
	hatpTaskList& operator=(hatpTaskList const&);

public:
	typedef hatpTaskMap::iterator iterator;
	typedef hatpTaskMap::const_iterator const_iterator;
	
	hatpTaskList() {}

	/** Tells if the list is empty.
	 * @return True if the list is empty, false otherwise
	 */
	bool empty() const;
	
	/** Add a task to the list.
	 * @param id Id of the task
	 * @param task Task to add
	 * @return True if it was possible to add the task, false otherwise
	 */
	bool addTask(hatpPlan& plan, unsigned int id, const hatp_msgs::Task& task);
	
	/** Check wether a given task exists or not.
	 * @param id Id of the task
	 * @return True if the task exists, false otherwise
	 */
	bool hasTask(unsigned int id) const;
	
	/** Get a task from its id
	 * @param id Task to return
	 * @return The task
	 */
	hatpTaskPtr getTask(unsigned int id);
	hatpTask& getTaskRef(unsigned int id);
	
	/** Get all the tasks available
	 * @return All the tasks
	 */
	hatpTaskMap& getTasks();
	hatpTaskMap const& getTasks() const;
	
	iterator begin();
	const_iterator begin() const;
	iterator end();
	const_iterator end() const;
	
	/** Get the list under a text form (for printing).
	 * @return A string containing the list under a text form
	 */
	std::string toString() const;
};

#endif //_HATPTASKLIST_H_
