#include <libhatp/hatpPlan.h>

#include <sstream>

hatpPlan::hatpPlan(const hatp_msgs::Plan& planMsg): id(planMsg.id), report(planMsg.report), tasks(NULL), tree(NULL), streams(NULL)
{
	if(isReportOK()){
		std::set<std::string> temp_agents;

		tasks=new hatpTaskList;
		for(std::vector<hatp_msgs::Task>::const_iterator it=planMsg.tasks.begin(); it!=planMsg.tasks.end(); ++it){
			tasks->addTask(*this, it->id, (*it));
			std::vector<std::string>::const_iterator it_agent=it->agents.begin();
			for(it_agent=it->agents.begin(); it_agent!=it->agents.end(); ++it_agent){
				temp_agents.insert(*it_agent);
			}
		}
		tree=new hatpTree(*this, planMsg);
		streams=new hatpStream(*this, planMsg.streams);
		
		std::set<std::string>::const_iterator it_agent;
		for(it_agent=temp_agents.begin(); it_agent!=temp_agents.end(); ++it_agent){
			agents.push_back(*it_agent);
		}
	}
}

hatpPlan::hatpPlan(const hatp_msgs::Plan::ConstPtr& planMsg):  id(planMsg->id), report(planMsg->report), tasks(NULL), tree(NULL), streams(NULL)
{
	if(isReportOK()){
		std::set<std::string> temp_agents;
		
		tasks=new hatpTaskList;
		for(std::vector<hatp_msgs::Task>::const_iterator it=planMsg->tasks.begin(); it!=planMsg->tasks.end(); ++it){
			tasks->addTask(*this, it->id, (*it));
			std::vector<std::string>::const_iterator it_agent=it->agents.begin();
			for(it_agent=it->agents.begin(); it_agent!=it->agents.end(); ++it_agent){
				temp_agents.insert(*it_agent);
			}
		}
		tree=new hatpTree(*this, *planMsg);
		streams=new hatpStream(*this, planMsg->streams);
		
		std::set<std::string>::const_iterator it_agent;
		for(it_agent=temp_agents.begin(); it_agent!=temp_agents.end(); ++it_agent){
			agents.push_back(*it_agent);
		}
	}
}

std::string hatpPlan::toString() const{
	std::ostringstream ss;
	
	ss<<"<----- PLAN ----->"<<std::endl;
	ss<<"Id: "<<id<<std::endl;
	ss<<"Report: "<<report<<std::endl;
	ss<<"IsValid: "<<std::boolalpha<<isValid()<<std::endl<<std::endl;
	if(isValid()){
		ss<<"<----- TASKS ----->"<<std::endl;
		ss<<tree->toString()<<std::endl<<std::endl;
// 		ss<<"<----- STREAMS ----->"<<std::endl;
// 		ss<<streams->toString();
	}
	
	return ss.str();
}

std::string hatpPlan::getReport() const{
	return report;
}

bool hatpPlan::isReportOK() const{
	return report=="OK";
}

bool hatpPlan::isValid() const{
	return isReportOK();
}

bool hatpPlan::hasTask(unsigned int id) const{
	return tasks->hasTask(id);
}

hatpTaskPtr hatpPlan::getTask(unsigned int id) const{
	return tasks->getTask(id);
}

hatpTaskList& hatpPlan::getTaskList(){
	return *tasks;
}

hatpTaskList const& hatpPlan::getTaskList() const{
	return *tasks;
}

std::vector<std::string> hatpPlan::getAgentsList() const{
	return agents;
}

hatpStream* hatpPlan::getStreams(){
	return streams;
}

hatpTree* hatpPlan::getTree(){
	return tree;
}

hatpPlan::~hatpPlan(){
	delete tasks;
	delete tree;
	delete streams;
}
