#include <libhatp/hatpTask.h>

#include <sstream>

#include <libhatp/hatpPlan.h>

//hatpTask::hatpTask(unsigned int id, TaskType type, std::string name, std::vector<std::string> agents,
//	std::vector<std::string > params, double cost, double start, double duration):
//	id(id), type(type), taskName(name), agents(agents), parameters(params), cost(cost),
//	startTime(start), duration(duration), endTime(start+duration),parent(0)
//{}

hatpTask::hatpTask(hatpPlan& plan, const hatp_msgs::Task& task):
	plan(plan),
	id(task.id), type((task.type==TYPE_ACTION?action:method)), taskName(task.name),
	agents(task.agents), parameters(task.parameters), cost(task.cost),
	startTime(((double)task.begin.sec)+(((double)task.begin.nsec)/1000000000.)),
	duration(((double)task.duration.sec)+(((double)task.duration.nsec)/1000000000.)),
	endTime(startTime+duration), parent(0)
{}

unsigned int hatpTask::getID() const{
	return id;
}

TaskType hatpTask::getType() const{
	return type;
}

bool hatpTask::isAction() const{
	return type==action;
}

bool hatpTask::isMethod() const{
	return type==method;
}

std::string hatpTask::getName() const{
	return taskName;
}

std::vector<std::string> hatpTask::getParameters() const{
	return parameters;
}

std::vector<std::string> hatpTask::getAgents() const{
	return agents;
}

double hatpTask::getCost() const{
	return cost;
}

double hatpTask::getStartTime() const{
	return startTime;
}

double hatpTask::getDuration() const{
	return duration;
}

double hatpTask::getEndTime() const{
	return endTime;
}

unsigned int hatpTask::getParent() const{
	return parent;
}

std::vector<unsigned int> hatpTask::getSubNodesIds() const{
	return subnodes;
}

hatpTaskSet hatpTask::getSubNodes() const{
	hatpTaskSet result;
	
	std::vector<unsigned int>::const_iterator it;
	
	for(it=subnodes.begin(); it!=subnodes.end(); ++it){
		if(plan.hasTask(*it))
			result.push_back(plan.getTask(*it));
	}
	return result;
}

std::vector<unsigned int> hatpTask::getTreePredecessorsIds() const{
	return treePredecessors;
}

hatpTaskSet hatpTask::getTreePredecessors() const{
	hatpTaskSet result;
	
	std::vector<unsigned int>::const_iterator it;
	
	for(it=treePredecessors.begin(); it!=treePredecessors.end(); ++it){
		if(plan.hasTask(*it))
			result.push_back(plan.getTask(*it));
	}
	return result;
}

std::vector<unsigned int> hatpTask::getTreeSuccessorsIds() const{
	return treeSuccessors;
}

hatpTaskSet hatpTask::getTreeSuccessors() const{
	hatpTaskSet result;
	
	std::vector<unsigned int>::const_iterator it;
	
	for(it=treeSuccessors.begin(); it!=treeSuccessors.end(); ++it){
		if(plan.hasTask(*it))
			result.push_back(plan.getTask(*it));
	}
	return result;
}

std::vector<unsigned int> hatpTask::getStreamPredecessorsIds() const{
	return streamPredecessors;
}

hatpTaskSet hatpTask::getStreamPredecessors() const{
	hatpTaskSet result;
	
	std::vector<unsigned int>::const_iterator it;
	
	for(it=streamPredecessors.begin(); it!=streamPredecessors.end(); ++it){
		if(plan.hasTask(*it))
			result.push_back(plan.getTask(*it));
	}
	return result;
}

std::vector<unsigned int> hatpTask::getStreamSuccessorsIds() const{
	return streamSuccessors;
}

hatpTaskSet hatpTask::getStreamSuccessors() const{
	hatpTaskSet result;
	
	std::vector<unsigned int>::const_iterator it;
	
	for(it=streamSuccessors.begin(); it!=streamSuccessors.end(); ++it){
		if(plan.hasTask(*it))
			result.push_back(plan.getTask(*it));
	}
	return result;
}

void hatpTask::setParent(unsigned int n){
	parent=n;
}

void hatpTask::addSubNode(unsigned int n){
	subnodes.push_back(n);
}

void hatpTask::addTreePredecessor(unsigned int n){
	treePredecessors.push_back(n);
}

void hatpTask::addTreeSuccessor(unsigned int n){
	treeSuccessors.push_back(n);
}

void hatpTask::addStreamPredecessor(unsigned int n){
	streamPredecessors.push_back(n);
}

void hatpTask::addStreamSuccessor(unsigned int n){
	streamSuccessors.push_back(n);
}

std::string hatpTask::getDescription() const{
	std::ostringstream result;
	result << "  " << taskName << "( ";
	for(unsigned int i = 0 ; i < parameters.size() ; i++ ) result << parameters[i] << " ";
	result << ")" << std::endl;
	return result.str();
}

std::string hatpTask::toString() const{
	std::ostringstream result;
	result << "NODE ID " << id << " (" << (type==action?"action":"method") << "):" << std::endl;
	result << "  " << taskName << "( ";
	for(unsigned int i = 0 ; i < parameters.size() ; i++ ) result << parameters[i] << " ";
	result << ")" << std::endl;
	result << "    agents = [";
	for(unsigned int i = 0 ; i < agents.size() ; i++ ) result << agents[i] << " ";
	result << "]" << std::endl;
	result << "    cost = " << cost << std::endl;
	result << "    time = {" << startTime << ", " << endTime << "}" << std::endl;
	result << "    parent = " << parent << std::endl;
	result << "    subnodes = [";
	for(unsigned int i = 0 ; i < subnodes.size() ; i++ ) result << subnodes[i] << " ";
	result << "]" << std::endl;
	result << "    (tree)predecessors = [";
	for(unsigned int i = 0 ; i < treePredecessors.size() ; i++ ) result << treePredecessors[i] << " ";
	result << "]" << std::endl;
	result << "    (tree)successors = [";
	for(unsigned int i = 0 ; i < treeSuccessors.size() ; i++ ) result << treeSuccessors[i] << " ";
	result << "]" << std::endl;
	result << "    (stream)predecessors = [";
	for(unsigned int i = 0 ; i < streamPredecessors.size() ; i++ ) result << streamPredecessors[i] << " ";
	result << "]" << std::endl;
	result << "    (stream)successors = [";
	for(unsigned int i = 0 ; i < streamSuccessors.size() ; i++ ) result << streamSuccessors[i] << " ";
	result << "]" << std::endl;
	return result.str();
}
