#include <libhatp/hatpTaskList.h>

#include <sstream>

#include <libhatp/hatpPlan.h>

bool hatpTaskList::empty() const{
	return tasks.empty();
}

bool hatpTaskList::addTask(hatpPlan& plan, unsigned int id, const hatp_msgs::Task& task){
	return tasks.insert(std::make_pair(id, std::make_shared<hatpTask>(plan, task))).second;
}

bool hatpTaskList::hasTask(unsigned int id) const{
	return(tasks.find(id)!=tasks.end());
}

hatpTaskPtr hatpTaskList::getTask(unsigned int id){
	iterator it=tasks.find(id);
	
	if(it==tasks.end()){
		std::stringstream idStr;
		idStr<<id;
		throw("ERROR: Task "+idStr.str()+" does not exist");
	}
	return it->second;
}

hatpTask& hatpTaskList::getTaskRef(unsigned int id){
	iterator it=tasks.find(id);
	
	if(it==tasks.end()){
		std::stringstream idStr;
		idStr<<id;
		throw("ERROR: Task "+idStr.str()+" does not exist");
	}
	return *(it->second);
}

hatpTaskMap& hatpTaskList::getTasks(){
	return tasks;
}

hatpTaskMap const& hatpTaskList::getTasks() const{
	return tasks;
}

hatpTaskList::iterator hatpTaskList::begin(){
	return tasks.begin();
}

hatpTaskList::const_iterator hatpTaskList::begin() const{
	return tasks.begin();
}

hatpTaskList::iterator hatpTaskList::end(){
	return tasks.end();
}

hatpTaskList::const_iterator hatpTaskList::end() const{
	return tasks.end();
}

std::string hatpTaskList::toString() const{
	std::ostringstream result;

	const_iterator it;
	for(it=tasks.begin(); it!=tasks.end(); ++it){
		result << it->second->toString() << std::endl;
	}
	return result.str();
}
