#include <libhatp/hatpTree.h>

#include <sstream>

#include <libhatp/hatpPlan.h>

// hatpTree::hatpTree(hatpPlan& plan): plan(plan),  {
// }

hatpTree::hatpTree(hatpPlan& plan, const hatp_msgs::Plan& msg): plan(plan), root(msg.treeRoot), depth(0) {
	std::vector<hatp_msgs::TreeNode>::const_iterator it;
	for(it=msg.tree.begin(); it!=msg.tree.end(); ++it){
		if(!plan.hasTask(it->taskId)){
			std::cout<<"WARNING: Unknown task id: "<<it->taskId<<std::endl;
			continue;
		}
		hatpTaskPtr task=plan.getTask(it->taskId);
		if(plan.hasTask(it->parent)) task->setParent(it->parent);
		
		std::vector<uint32_t>::const_iterator it_ids;
		for(it_ids=it->subnodes.begin(); it_ids!=it->subnodes.end(); ++it_ids){
			if(!plan.hasTask(*it_ids)) continue;
			task->addSubNode(*it_ids);
		}
		for(it_ids=it->predecessors.begin(); it_ids!=it->predecessors.end(); ++it_ids){
			if(!plan.hasTask(*it_ids)) continue;
			task->addTreePredecessor(*it_ids);
		}
		for(it_ids=it->successors.begin(); it_ids!=it->successors.end(); ++it_ids){
			if(!plan.hasTask(*it_ids)) continue;
			task->addTreeSuccessor(*it_ids);
		}
	}
	
	//Compute the tree depth with a recursive lamba function
	std::function<void(hatpTaskPtr, unsigned int)> computeTreeDepth = [&computeTreeDepth, &plan, this](hatpTaskPtr task, unsigned int level) {
		hatpTaskSet temp=task->getSubNodes();
		if(temp.empty()) return;
		
		++level;
		if(level>depth) depth=level;
		
		for(hatpTaskSet::const_iterator it=temp.begin(); it!=temp.end(); ++it){
			computeTreeDepth(*it, level);
		}
	};
	
	computeTreeDepth(getRootNode(), 0);
}

unsigned int hatpTree::getRootID() const{
	return root;
}

hatpTaskPtr hatpTree::getRootNode() const{
	if(!plan.hasTask(root)){
		return NULL;
	}
	
	return plan.getTask(root);
}

unsigned int hatpTree::getTreeDepth() const{
	return depth;
}

std::string hatpTree::toString() const{
	std::ostringstream ss;
	
	ss<<plan.getTaskList().toString()<<std::endl;
	
	return ss.str();
}